<?php
namespace Utility\Entity;

use Doctrine\ORM\Mapping as ORM;



/**
 * Notification logs.
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 * @ORM\Table(name="lib_api_log")
 */
class ApiLog
{

	/**
	 * Can archive records.
	 */
	const ARCHIVE = true;
	/**
	 * Pull Synchronization Strategy for this table.
	 */
	const PULL_SYNCH_STRATEGY = false;
	/**
	 * Push Synchronization Strategy for this table.
	 */
	const PUSH_SYNCH_STRATEGY = false;

	/**
	 * @ORM\Id
	 * @ORM\Column(type="integer");
	 * @ORM\GeneratedValue(strategy="AUTO")
	 */
	protected $id;

	/**
	 * @ORM\Column(type="string", nullable=true, length=250)
	 */
	protected $service;

	/**
	 * @ORM\Column(type="array", nullable=true)
	 */
	protected $request;

	/**
	 * @ORM\Column(type="array", nullable=true)
	 */
	protected $response;

	/**
	 * @ORM\Column(type="string", length=30, name="status")
	 */
	protected $status;

	/**
	 * @ORM\ManyToOne(targetEntity="\Company\Entity\Company")
	 * @ORM\JoinColumn(nullable=true, name="from_company_id")
	 **/
	protected $fromCompany;

	/**
	 * @ORM\ManyToOne(targetEntity="\User\Entity\Profile")
	 * @ORM\JoinColumn(nullable=true, name="from_profile_id")
	 **/
	protected $fromProfile;

	/**
	 * @ORM\Column(type="datetime");
	 */
	protected $created;

	/**
	 * @ORM\Column(type="datetime", nullable=true);
	 */
	protected $updated;

	/**
	 * @ORM\Column(type="boolean");
	 */
	protected $archived = false;


	/**
	 * Magic getter to expose protected properties.
	 * @param string $property
	 * @return mixed
	 */
	public function __get($property)
	{
		return $this->$property;
	}

	/**
	 * Magic setter to save protected properties.
	 * @param string $property
	 * @param mixed $value
	 */
	public function __set($property, $value)
	{
		$this->$property = $value;
	}

	/**
	 * @ORM\PrePersist
	 */
	public function setCreateTime()
	{
		$this->created = new \DateTime("now");
	}

	/**
	 * @ORM\PreUpdate
	 */
	public function setUpdateTime()
	{
		$this->updated = new \DateTime("now");
	}

	/**
	 * Convert the object to an array.
	 * @param array $expand
	 * @param array $intersect
	 * @return array
	 */
	public function toArray(array $expand = array('attachment'), array $intersect = array())
	{
		$dateTimeFormat = \Utility\Registry::getConfigParam('DateTimeFormat');
		$includeAll = empty($intersect);
		$data = array();
		($includeAll || isset($intersect['id']))
			&& $data['id'] = $this->id;
		($includeAll || isset($intersect['service']))
			&& $data['service'] = $this->service;
		($includeAll || isset($intersect['request']))
			&& $data['request'] = $this->request;
		($includeAll || isset($intersect['response']))
			&& $data['response'] = $this->response;
		($includeAll || isset($intersect['status']))
			&& $data['status'] = $this->status;
		($includeAll || isset($intersect['fromCompany']))
			&& $data['fromCompany'] = in_array('fromCompany', $expand)
									&& !is_null($this->fromCompany)
				? $this->fromCompany->toArray($expand, $intersect)
				: null;
		($includeAll || isset($intersect['fromProfile']))
			&& $data['fromProfile'] = in_array('fromProfile', $expand)
									&& !is_null($this->fromProfile)
				? $this->fromProfile->toArray($expand, $intersect)
				: null;
		($includeAll || isset($intersect['created']))
			&& $data['created'] = !is_null($this->created)
				? $this->created->format($dateTimeFormat)
				: null;
		($includeAll || isset($intersect['updated']))
			&& $data['updated'] = !is_null($this->updated)
				? $this->updated->format($dateTimeFormat)
				: null;
		return $data;
	}

	/**
	 * Populate from an array.
	 * @param array $data
	 */
	public function fromArray($data = array())
	{
		isset($data['id'])
			&& $this->id = $data['id'];
		isset($data['service'])
			&& $this->service = $data['service'];
		isset($data['request'])
			&& $this->request = $data['request'];
		isset($data['response'])
			&& $this->response = $data['response'];
		isset($data['status'])
			&& $this->status = $data['status'];
		isset($data['fromCompany'])
			&& $this->fromCompany = $data['fromCompany'];
		isset($data['fromProfile'])
			&& $this->fromProfile = $data['fromProfile'];
	}

}
