<?php
namespace Utility;



/**
 * Simple event handler.
 * @author andre.fourie
 *
 */
class Event
{

	/**
	 * @var array
	 */
	static protected $listener = array();


	/**
	 * Register an event listener.
	 * @param string $eventName
	 * @param object|string $class
	 * @param string $action
	 */
	static public function listen($eventName, $class, $action)
	{
		!isset(self::$listener[$eventName])
			&& self::$listener[$eventName] = array();
		self::$listener[$eventName][] = array(
				'Class'  => $class,
				'Action' => $action
		);
	}

	/**
	 * Trigger an event.
	 * @param string $eventName
	 * @param unknown|null $data
	 */
	static public function trigger($eventName, $data = null)
	{
		if (!isset(self::$listener[$eventName]))
		{
			return;
		}
		foreach (self::$listener[$eventName] as $listener)
		{
			$obj = is_object($listener['Class'])
				? $listener['Class']
				: new $listener['Class']();
			$action = $listener['Action'];
			$obj->$action($eventName, $data);
		}
	}

}

