<?php
namespace Utility\Export;

class Csv
{
	private $_file;
	private $_fileHandle;
	private $_delimiter = ',';
	private $_enclosure = '"';
	private $_escape = '\\';
	private $_recordLength = 0;

	const WRITE_OVERWRITE  	= 1;
	const WRITE_APPEND 		= 2;

	public function __construct($file, $writeMethod = self::WRITE_OVERWRITE)
	{
		$this->_file = $file;
		$method = self::WRITE_OVERWRITE == $writeMethod
			? 'w'
			: 'a';
		$this->_fileHandle = fopen($this->_file, $method);
		if (!$this->_fileHandle)
		{
			throw new \Exception('File ' . $this->_file . ' could not be opened for writing');
		}
	}

	public function setDelimiter($delimiter)
	{
		$this->_delimiter = $delimiter;
		if (!empty($this->_headers))
		{
			$this->reset();
			$this->_headers = $this->getRecord(self::FETCH_LAZY);
		}
	}

	public function setEnclosure($enclosure)
	{
		$this->_enclosure = $enclosure;
	}

	public function setEscape($escape)
	{
		$this->_escape = $escape;
	}

	public function setRecordLength($length)
	{
		$this->_recordLength = $length;
	}

	/**
	 * Writes a record to the file
	 * @return array
	 */
	public function putRecord($record)
	{
		$bytesWritten = fputcsv($this->_fileHandle, $record, $this->_delimiter, $this->_enclosure);
		return 0 < $bytesWritten
			? true
			: false;
	}

	public function __destruct()
	{
		if ($this->_fileHandle)
		{
			fclose($this->_fileHandle);
		}
	}
}
